@@ -98,7 +98,7 @@
 #include <string.h>
 
 /* Debugging category */
-GST_DEBUG_CATEGORY (dfbvideosink_debug);
+GST_DEBUG_CATEGORY_STATIC (dfbvideosink_debug);
 #define GST_CAT_DEFAULT dfbvideosink_debug
 
 /* ElementFactory information */
@@ -1425,7 +1425,13 @@ gst_dfbvideosink_change_state (GstElement * element, GstStateChange transition)
     case GST_STATE_CHANGE_NULL_TO_READY:
       dfbvideosink->running = TRUE;
       if (!dfbvideosink->setup) {
-        gst_dfbvideosink_setup (dfbvideosink);
+        if (!gst_dfbvideosink_setup (dfbvideosink)) {
+          GST_DEBUG_OBJECT (dfbvideosink, "setup failed when changing state "
+              "from NULL to READY");
+          GST_ELEMENT_ERROR (dfbvideosink, RESOURCE, OPEN_WRITE,
+              (NULL), ("Failed initializing DirectFB system"));
+          return GST_STATE_CHANGE_FAILURE;
+        }
       }
       break;
     case GST_STATE_CHANGE_READY_TO_PAUSED:
@@ -1697,6 +1703,12 @@ gst_dfbvideosink_buffer_alloc (GstBaseSink * bsink, guint64 offset, guint size,
   GST_DEBUG_OBJECT (dfbvideosink, "a buffer of %d bytes was requested "
       "with caps %" GST_PTR_FORMAT " and offset %llu", size, caps, offset);
 
+  if (G_UNLIKELY (!dfbvideosink->setup)) {
+    GST_DEBUG_OBJECT (dfbvideosink, "we are not setup yet, can't allocate!");
+    *buf = NULL;
+    return ret;
+  }
+
   desired_caps = gst_caps_copy (caps);
 
   structure = gst_caps_get_structure (desired_caps, 0);
@@ -1930,7 +1942,6 @@ gst_dfbvideosink_navigation_send_event (GstNavigation * navigation,
   GstDfbVideoSink *dfbvideosink = GST_DFBVIDEOSINK (navigation);
   GstEvent *event;
   GstVideoRectangle src, dst, result;
-  gint width, height;
   double x, y;
   GstPad *pad = NULL;
 
@@ -2213,6 +2224,8 @@ gst_dfbvideosink_finalize (GObject * object)
   if (dfbvideosink->setup) {
     gst_dfbvideosink_cleanup (dfbvideosink);
   }
+
+  G_OBJECT_CLASS (parent_class)->finalize (object);
 }
 
 static void
@@ -2354,6 +2367,11 @@ gst_dfbvideosink_get_type (void)
       NULL,
       NULL,
     };
+    static const GInterfaceInfo colorbalance_info = {
+      (GInterfaceInitFunc) gst_dfbvideosink_colorbalance_init,
+      NULL,
+      NULL,
+    };
 
     dfbvideosink_type = g_type_register_static (GST_TYPE_VIDEO_SINK,
         "GstDfbVideoSink", &dfbvideosink_info, 0);
@@ -2362,6 +2380,8 @@ gst_dfbvideosink_get_type (void)
         GST_TYPE_IMPLEMENTS_INTERFACE, &iface_info);
     g_type_add_interface_static (dfbvideosink_type, GST_TYPE_NAVIGATION,
         &navigation_info);
+    g_type_add_interface_static (dfbvideosink_type, GST_TYPE_COLOR_BALANCE,
+        &colorbalance_info);
   }
 
   return dfbvideosink_type;
@@ -2370,7 +2390,7 @@ gst_dfbvideosink_get_type (void)
 static gboolean
 plugin_init (GstPlugin * plugin)
 {
-  if (!gst_element_register (plugin, "dfbvideosink", GST_RANK_PRIMARY,
+  if (!gst_element_register (plugin, "dfbvideosink", GST_RANK_MARGINAL,
           GST_TYPE_DFBVIDEOSINK))
     return FALSE;
 
