@@ -45,17 +45,10 @@ GST_ELEMENT_DETAILS ("DTS audio decoder",
     "Decodes DTS audio streams",
     "Ronald Bultje <rbultje@ronald.bitfreak.net>");
 
-enum
-{
-  /* FILL ME */
-  LAST_SIGNAL
-};
-
 enum
 {
   ARG_0,
   ARG_DRC
-      /* FILL ME */
 };
 
 static GstStaticPadTemplate sink_factory = GST_STATIC_PAD_TEMPLATE ("sink",
@@ -153,18 +146,14 @@ gst_dtsdec_class_init (GstDtsDecClass * klass)
 static void
 gst_dtsdec_init (GstDtsDec * dtsdec, GstDtsDecClass * g_class)
 {
-  /* create the sink and src pads */
-  dtsdec->sinkpad =
-      gst_pad_new_from_template (gst_static_pad_template_get
-      (&sink_factory), "sink");
-  gst_pad_set_chain_function (dtsdec->sinkpad, gst_dtsdec_chain);
+  dtsdec->sinkpad = gst_pad_new_from_static_template (&sink_factory, "sink");
+  gst_pad_set_chain_function (dtsdec->sinkpad,
+      GST_DEBUG_FUNCPTR (gst_dtsdec_chain));
   gst_pad_set_event_function (dtsdec->sinkpad,
       GST_DEBUG_FUNCPTR (gst_dtsdec_sink_event));
   gst_element_add_pad (GST_ELEMENT (dtsdec), dtsdec->sinkpad);
 
-  dtsdec->srcpad =
-      gst_pad_new_from_template (gst_static_pad_template_get
-      (&src_factory), "src");
+  dtsdec->srcpad = gst_pad_new_from_static_template (&src_factory, "src");
   gst_pad_use_fixed_caps (dtsdec->srcpad);
   gst_element_add_pad (GST_ELEMENT (dtsdec), dtsdec->srcpad);
 
@@ -308,8 +297,7 @@ gst_dtsdec_sink_event (GstPad * pad, GstEvent * event)
   GstDtsDec *dtsdec = GST_DTSDEC (gst_pad_get_parent (pad));
   gboolean ret = FALSE;
 
-  GST_LOG ("Handling event of type %d timestamp %llu", GST_EVENT_TYPE (event),
-      GST_EVENT_TIMESTAMP (event));
+  GST_LOG_OBJECT (dtsdec, "%s event", GST_EVENT_TYPE_NAME (event));
 
   switch (GST_EVENT_TYPE (event)) {
     case GST_EVENT_NEWSEGMENT:{
@@ -331,14 +319,6 @@ gst_dtsdec_sink_event (GstPad * pad, GstEvent * event)
       ret = gst_pad_event_default (pad, event);
       break;
     }
-    case GST_EVENT_TAG:
-    case GST_EVENT_EOS:{
-      ret = gst_pad_event_default (pad, event);
-      break;
-    }
-    case GST_EVENT_FLUSH_START:
-      ret = gst_pad_event_default (pad, event);
-      break;
     case GST_EVENT_FLUSH_STOP:
       if (dtsdec->cache) {
         gst_buffer_unref (dtsdec->cache);
@@ -392,6 +372,45 @@ gst_dtsdec_handle_frame (GstDtsDec * dts, guint8 * data,
     gst_dtsdec_update_streaminfo (dts);
   }
 
+  if (dts->request_channels == DTS_CHANNEL) {
+    GstCaps *caps;
+
+    caps = gst_pad_get_allowed_caps (dts->srcpad);
+    if (caps && gst_caps_get_size (caps) > 0) {
+      GstCaps *copy = gst_caps_copy_nth (caps, 0);
+      GstStructure *structure = gst_caps_get_structure (copy, 0);
+      gint channels;
+      const int dts_channels[6] = {
+        DTS_MONO,
+        DTS_STEREO,
+        DTS_STEREO | DTS_LFE,
+        DTS_2F2R,
+        DTS_2F2R | DTS_LFE,
+        DTS_3F2R | DTS_LFE,
+      };
+
+      /* Prefer the original number of channels, but fixate to something 
+       * preferred (first in the caps) downstream if possible.
+       */
+      gst_structure_fixate_field_nearest_int (structure, "channels",
+          flags ? gst_dtsdec_channels (flags, NULL) : 6);
+      gst_structure_get_int (structure, "channels", &channels);
+      if (channels <= 6)
+        dts->request_channels = dts_channels[channels - 1];
+      else
+        dts->request_channels = dts_channels[5];
+
+      gst_caps_unref (copy);
+    } else if (flags) {
+      dts->request_channels = dts->stream_channels;
+    } else {
+      dts->request_channels = DTS_3F2R | DTS_LFE;
+    }
+
+    if (caps)
+      gst_caps_unref (caps);
+  }
+
   /* process */
   flags = dts->request_channels | DTS_ADJUST_LEVEL;
   dts->level = 1;
@@ -435,10 +454,8 @@ gst_dtsdec_handle_frame (GstDtsDec * dts, guint8 * data,
     result = gst_pad_alloc_buffer_and_set_caps (dts->srcpad, 0,
         (SAMPLE_WIDTH / 8) * 256 * num_c, GST_PAD_CAPS (dts->srcpad), &out);
 
-    if (result != GST_FLOW_OK) {
-      GST_ELEMENT_ERROR (dts, RESOURCE, FAILED, (NULL), ("Out of memory"));
-      goto done;
-    }
+    if (result != GST_FLOW_OK)
+      break;
 
     GST_BUFFER_TIMESTAMP (out) = dts->current_ts;
     GST_BUFFER_DURATION (out) = GST_SECOND * 256 / dts->sample_rate;
@@ -457,16 +474,10 @@ gst_dtsdec_handle_frame (GstDtsDec * dts, guint8 * data,
     /* push on */
     result = gst_pad_push (dts->srcpad, out);
 
-    if (result != GST_FLOW_OK) {
-      gst_buffer_unref (out);
-      goto done;
-    }
-
-
+    if (result != GST_FLOW_OK)
+      break;
   }
 
-done:
-
   return result;
 }
 
@@ -475,7 +486,7 @@ gst_dtsdec_chain (GstPad * pad, GstBuffer * buf)
 {
   GstDtsDec *dts;
   guint8 *data;
-  gint64 size;
+  gint size;
   gint length, flags, sample_rate, bit_rate, frame_length;
   GstFlowReturn result = GST_FLOW_OK;
 
@@ -547,7 +558,7 @@ gst_dtsdec_change_state (GstElement * element, GstStateChange transition)
       dts->sample_rate = -1;
       dts->stream_channels = 0;
       /* FIXME force stereo for now */
-      dts->request_channels = DTS_STEREO;
+      dts->request_channels = DTS_CHANNEL;
       dts->using_channels = 0;
       dts->level = 1;
       dts->bias = 0;
